#include "MainWindow.h"
#include "Settings.h"
#include "System.h"
#include "FilePaths.h"

MainWindow::MainWindow(QWidget *parent) : QWidget(parent)
{
	setFixedSize(System::getSize());

	// Main Layout
	QVBoxLayout *mvLayout = new QVBoxLayout();

	// Splitted in 4 vertical parts
	QGridLayout *mgLayout = new QGridLayout();
	//QTabWidget *tabWidget = new QTabWidget();
	QGridLayout *mgTabs = new QGridLayout();
	QStackedWidget *stackedWidget = new QStackedWidget();
	QHBoxLayout *mhLayout = new QHBoxLayout();

	// Part 1
	pathToBinary = new QLineEdit();
	pathToBinary -> setReadOnly(true);
	mgLayout -> addWidget(pathToBinary, 0, 0, 0, 3);
	QPushButton *browseButton = new QPushButton("Browse");
	mgLayout -> addWidget(browseButton, 0, 3);
	connect(browseButton, SIGNAL(clicked()), this, SLOT(browseButtonClicked()));

	// Part 2
	QRadioButton *welcomeButton = new QRadioButton("Welcome");
	QRadioButton *featuresButton = new QRadioButton("Features");
	QRadioButton *modButton = new QRadioButton("MOD");
	QRadioButton *videoButton = new QRadioButton("Video");
	QRadioButton *audiojoyButton = new QRadioButton("Audio/Joy");
	QRadioButton *speechButton = new QRadioButton("Network");
	QRadioButton *networkButton = new QRadioButton("Speech");
	QRadioButton *registryButton = new QRadioButton("Registry");
	mgTabs -> addWidget(welcomeButton, 0, 0);
	mgTabs -> addWidget(featuresButton, 0, 1);
	mgTabs -> addWidget(modButton, 0, 2);
	mgTabs -> addWidget(videoButton, 0, 3);
	mgTabs -> addWidget(audiojoyButton, 1, 0);
	mgTabs -> addWidget(speechButton, 1, 1);
	mgTabs -> addWidget(networkButton, 1, 2);
	mgTabs -> addWidget(registryButton, 1, 3);

	QButtonGroup *buttonGroup = new QButtonGroup(this);
	buttonGroup -> addButton(welcomeButton, 0);
	buttonGroup -> addButton(featuresButton, 1);
	buttonGroup -> addButton(modButton, 2);
	buttonGroup -> addButton(videoButton, 3);
	buttonGroup -> addButton(audiojoyButton, 4);
	buttonGroup -> addButton(speechButton, 5);
	buttonGroup -> addButton(networkButton, 6);
	buttonGroup -> addButton(registryButton, 7);

	// Part 3
	tabAudioJoy = new TabAudioJoy();
	tabFeatures = new TabFeatures();
	tabMod = new TabMod();
	tabNetwork = new TabNetwork();
	tabRegistry = new TabRegistry();
	tabSpeech = new TabSpeech();
	tabVideo = new TabVideo();
	tabWelcome = new TabWelcome();
	/*
	tabWidget -> addTab(tabWelcome, "Welcome");
	tabWidget -> addTab(tabFeatures, "Features");
	tabWidget -> addTab(tabMod, "MOD");
	tabWidget -> addTab(tabVideo, "Video");
	tabWidget -> addTab(tabAudioJoy, "Audio/Joystick");
	tabWidget -> addTab(tabSpeech, "Speech");
	tabWidget -> addTab(tabNetwork, "Network");
	tabWidget -> addTab(tabRegistry, "Registry");
	*/
	stackedWidget -> addWidget(tabWelcome);
	stackedWidget -> addWidget(tabFeatures);
	stackedWidget -> addWidget(tabMod);
	stackedWidget -> addWidget(tabVideo);
	stackedWidget -> addWidget(tabAudioJoy);
	stackedWidget -> addWidget(tabSpeech);
	stackedWidget -> addWidget(tabNetwork);
	stackedWidget -> addWidget(tabRegistry);

	connect(buttonGroup, SIGNAL(buttonPressed(int)), stackedWidget, SLOT(setCurrentIndex(int)));
	welcomeButton -> click();
	

	// Part 4
	QPushButton *runButton = new QPushButton(tr("Run"));
	QPushButton *okButton = new QPushButton(tr("OK"));
	QPushButton *cancelButton = new QPushButton(tr("Cancel"));
	QPushButton *applyButton = new QPushButton(tr("Apply"));
	mhLayout -> addWidget(runButton);
	mhLayout -> addWidget(okButton);
	mhLayout -> addWidget(cancelButton);
	mhLayout -> addWidget(applyButton);

	connect(runButton, SIGNAL(clicked()), this, SLOT(runButtonClicked()));
	connect(okButton, SIGNAL(clicked()), this, SLOT(okButtonClicked()));
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancelButtonClicked()));
	connect(applyButton, SIGNAL(clicked()), this, SLOT(applyButtonClicked()));

	// Connect them together
	mvLayout -> addLayout(mgLayout);
	mvLayout -> addLayout(mgTabs);
	mvLayout -> addWidget(stackedWidget);
	mvLayout -> addLayout(mhLayout);
	setLayout(mvLayout);

	QShortcut *enter = new QShortcut(QKeySequence(Qt::Key_Return), this);
	connect(enter, SIGNAL(activated()), okButton, SLOT(click()));

	okButton -> setFocus();

	updateUI();

	// tabs that can update whole UI
	connect( tabMod, SIGNAL( setUpdateUI() ), this, SLOT( updateUI() ) );
}

void MainWindow::updateUI()
{
	pathToBinary -> setText( SETTINGS.getExeFilePath() );
	tabFeatures -> updateUI();
	tabMod -> updateUI();
}

void MainWindow::browseButtonClicked()
{
	QString path = QFileDialog::getOpenFileName(this, "Select path to game executable", FILEPATHS.getGameDir());
	if(!path.isEmpty())
		SETTINGS.setExeFilePath(path);
	updateUI();
}

void MainWindow::runButtonClicked()
{
	SETTINGS.saveConfigs();
	/* if GAMEDIR is currentPath this does nothing, so its safe */
	System::chDir( FILEPATHS.getGameDir() );
	System::run( SETTINGS.getExeFilePath() );
}

void MainWindow::okButtonClicked()
{
	SETTINGS.saveConfigs();
	qApp->quit();
}

void MainWindow::cancelButtonClicked()
{
	qApp->quit();
}

void MainWindow::applyButtonClicked()
{
	SETTINGS.saveConfigs();
}
