#include "TabFeatures.h"
#include "Settings.h"

MyCheckBox::MyCheckBox(const QString &flag, const QString &desc, QWidget *parent) : QCheckBox(parent)
{
	this -> flag = flag;
	this -> desc = desc;
	if(desc.isEmpty())
		setText(flag);
	else
		setText(desc);

	connect(this, SIGNAL(stateChanged(int)), this, SLOT(emitMyStateChanged(int)));
}

void MyCheckBox::emitMyStateChanged(int state)
{
	emit myStateChanged(state, flag);
}

TabFeatures::TabFeatures(QWidget *parent) : QWidget(parent)
{
	stackedWidget = NULL;

	QVBoxLayout *mainLayout = new QVBoxLayout();
	QGroupBox *groupBox = new QGroupBox();

	inGroupLayout = new QVBoxLayout();

	flagsTextEdit = new QTextEdit();

	QGridLayout *gridLayout = new QGridLayout();
	QLabel *easySetupLabel = new QLabel();
	QLabel *customFlagsLabel = new QLabel();
	QLabel *listTypeLabel = new QLabel();
	easySetupComboBox = new QComboBox();
	customFlagsLineEdit = new QLineEdit();
	listComboBox = new QComboBox();

	QLabel *featureLabel = new QLabel();

	QFont smallerFont;
	smallerFont.setPointSize(8);
	flagsTextEdit -> setFont(smallerFont);
	flagsTextEdit -> setReadOnly(true);
	flagsTextEdit -> setFixedHeight(110);
	//flagsTextEdit -> setWordWrapMode( QTextOption::WordWrap );

	easySetupLabel -> setText("Easy setup");
	customFlagsLabel -> setText("Custom flags");
	listTypeLabel -> setText("List type");
	gridLayout -> addWidget(easySetupLabel, 0, 0);
	gridLayout -> addWidget(customFlagsLabel, 1, 0);
	gridLayout -> addWidget(listTypeLabel, 2, 0);
	gridLayout -> addWidget(easySetupComboBox, 0, 1);
	gridLayout -> addWidget(customFlagsLineEdit, 1, 1);
	gridLayout -> addWidget(listComboBox, 2, 1);

	featureLabel -> setText("<font color=red>Double click flag for online documentation on the feature</font>");
	smallerFont.setPointSize(7);
	featureLabel -> setFont(smallerFont);

	inGroupLayout -> addWidget(flagsTextEdit);
	inGroupLayout -> addLayout(gridLayout);
	//inGroupLayout -> addWidget(featureLabel);
	groupBox -> setLayout(inGroupLayout);

	mainLayout -> addWidget(groupBox);
	mainLayout -> setContentsMargins(0, 0, 0, 0);
	setLayout(mainLayout);

	drawCheckBoxList();
}

void TabFeatures::updateUI()
{
	/* flags text list */
	flagsTextEdit -> setText(SETTINGS.generateCmdLine());

	/* combo box with easy flags sets */
	easySetupComboBox -> clear();
	for (int i = 0; i < SETTINGS.getEasyFlagSets().size(); i++)
		easySetupComboBox -> insertItem(i, SETTINGS.getEasyFlagSets().at(i));

	/* custom flags text edit */
	customFlagsLineEdit -> setText( SETTINGS.getCustomFlags() );

	/* combo box with flag types */
	listComboBox -> clear();
	for (int i = 0; i < SETTINGS.getFlagTypes().size(); i++)
		listComboBox -> insertItem(i, SETTINGS.getFlagTypes().at(i));

	/* flags lists */
	drawCheckBoxList();

	/* lets (re)connect things */
	connect(easySetupComboBox, SIGNAL(activated(int)), this, SLOT(selectEasyFlagSet(int)));
	connect(customFlagsLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(updateCustomFlags(const QString&)));
}

void TabFeatures::drawCheckBoxList()
{
	if( stackedWidget != NULL )
	{
		inGroupLayout -> removeWidget( stackedWidget );
		delete stackedWidget;
	}

	stackedWidget = new QStackedWidget();

	for (int i = 0; i < SETTINGS.getFlagTypes().size(); i++)
	{
		QScrollArea *scrollArea = new QScrollArea();
		QWidget *flagsList = new QWidget();
		QVBoxLayout *boxLayout = new QVBoxLayout();

		for( int j = 0; j < SETTINGS.getFlags().size(); j++ )
		{
			const CFlag cfFlag = SETTINGS.getFlags().at( j );
			if( cfFlag.m_qstrType == SETTINGS.getFlagTypes().at( i ) )
			{
				MyCheckBox *checkBox = new MyCheckBox( cfFlag.m_qstrFlag, cfFlag.m_qstrDesc );
				if( cfFlag.m_bState )
					checkBox -> setChecked( true );
				connect(checkBox, SIGNAL(myStateChanged(int, const QString&)), this, SLOT(setFlag(int, const QString&)));
				boxLayout -> addWidget(checkBox);
			}
		}

		boxLayout -> setContentsMargins(0, 0, 0, 0);
		boxLayout -> setSpacing(0);

		QFont smallerFont;
		smallerFont.setPointSize(8);
		flagsList -> setFont(smallerFont);
		flagsList -> setLayout(boxLayout);

		scrollArea -> setBackgroundRole(QPalette::Light);
		scrollArea -> setWidget(flagsList);

		stackedWidget -> addWidget(scrollArea);
	}
	inGroupLayout -> addWidget(stackedWidget);

	/* lets (re)connect things */
	connect(listComboBox, SIGNAL(currentIndexChanged(int)), stackedWidget, SLOT(setCurrentIndex(int)));
	listComboBox -> setCurrentIndex(0);
}

void TabFeatures::selectEasyFlagSet( int nType )
{
	SETTINGS.setEasyTypeSet( nType );
	drawCheckBoxList();
	flagsTextEdit -> setText( SETTINGS.generateCmdLine() );
}

void TabFeatures::updateCustomFlags( const QString & flags )
{
	SETTINGS.setCustomFlags(flags);
	flagsTextEdit -> setText( SETTINGS.generateCmdLine() );
}

void TabFeatures::setFlag(int state, const QString& flag)
{
	SETTINGS.setFlag(flag, state);
	flagsTextEdit -> setText( SETTINGS.generateCmdLine() );
}
