#include "TabWelcome.h"
#include "System.h"

QPixmap GameInfo::getGamePixmap()
{
#ifdef BTRL
	return QPixmap(":title_btrl.bmp");
#else
#ifdef TBP
	return QPixmap(":title_tbp.bmp");
#else
#ifdef WCSAGA
	return QPixmap(":title_wcsaga.bmp");
#else
	System::randomize();
	int titleNumber = System::random(1, 3);

	switch(titleNumber) {
		case 1:
			return QPixmap(":title_fs2_1.bmp");
			break;
		case 2:
			return QPixmap(":title_fs2_2.bmp");
			break;
		case 3:
			return QPixmap(":title_fs2_3.bmp");
			break;
		default:
			return QPixmap();
	}
#endif /* WCSAGA */
#endif /* TBP */
#endif /* BTRL */
}

QString GameInfo::getWelcomeText()
{
#ifdef BTRL
	return "Welcome to Beyond The Red Line";
#else
#ifdef TBP
	return "Welcome to The Babylon Project";
#else
#ifdef WCSAGA
	return "Welcome to Wing Commander Saga";
#else /* FS2_OPEN */
	return "Welcome to FreeSpace2 Open";
#endif /* WCSAGA */
#endif /* TBP */
#endif /* BTRL */
}

QString GameInfo::getReadme()
{
#ifdef BTRL
	return "http://www.game-warden.com/bsg/about.html";
#else
#ifdef TBP
	return "http://babylon.hard-light.net/r3_readme.php";
#else
#ifdef WCSAGA
	return "http://www.wcsaga.com/";
#else /* FS2_OPEN */
	return "http://www.hard-light.net/wiki/index.php/Main_Page";
#endif /* WCSAGA */
#endif /* TBP */
#endif /* BTRL */
}

QString GameInfo::getForums()
{
#ifdef BTRL
	return "http://www.game-warden.com/forum/forumdisplay.php?f=36";
#else
#ifdef TBP
	return "http://www.hard-light.net/forums/index.php/board,105.0.html";
#else
#ifdef WCSAGA
	return "http://www.hard-light.net/forums/index.php/board,46.0.html";
#else /* FS2_OPEN */
	return "http://www.hard-light.net/forums/index.php/board,50.0.html";
#endif /* WCSAGA */
#endif /* TBP */
#endif /* BTRL */
}

QString GameInfo::getReportBug()
{
#ifdef BTRL
	return "http://scp.indiegames.us/mantis/main_page.php";
#else
#ifdef TBP
	return "http://scp.indiegames.us/mantis/main_page.php";
#else
#ifdef WCSAGA
	return "http://scp.indiegames.us/mantis/main_page.php";
#else /* FS2_OPEN */
	return "http://scp.indiegames.us/mantis/main_page.php";
#endif /* WCSAGA */
#endif /* TBP */
#endif /* BTRL */
}

TabWelcome::TabWelcome(QWidget *parent) : QWidget(parent)
{
	QVBoxLayout *mainLayout = new QVBoxLayout();
	QGroupBox *groupBox = new QGroupBox();

	QVBoxLayout *inGroupLayout = new QVBoxLayout();

	QLabel *titleLabel = new QLabel();
	QLabel *textLabel1 = new QLabel();
	QLabel *textLabel2 = new QLabel();

	QGroupBox *troubleGroupBox = new QGroupBox("Troubleshooting");
	QVBoxLayout *troubleVBoxLayout = new QVBoxLayout();
	QLabel *troubleLabel = new QLabel();
	QHBoxLayout *troubleHBoxLayout = new QHBoxLayout();
	QPushButton *troubleReadmeButton = new QPushButton("Readme");
	QPushButton *troubleForumsButton = new QPushButton("Forums");
	QPushButton *troubleReportBugButton = new QPushButton("Report Bug");

	titleLabel -> setFixedSize(278, 57);
	titleLabel -> setFrameStyle(QFrame::StyledPanel);

	titleLabel -> setPixmap(GameInfo::getGamePixmap());
	textLabel1 -> setText(GameInfo::getWelcomeText());

	QFont smallerFont;
	smallerFont.setPointSize(8);
	textLabel2 -> setFont(smallerFont);
	textLabel2 -> setWordWrap(true);
	textLabel2 -> setText("Basic setup: The first time you run the launcher you must use browser "
			"to find your fs2_open executable, then press \"Apply\". From now on you "
			"will be able to run fs2_open without loading the launcher.");

	troubleHBoxLayout -> addWidget(troubleReadmeButton);
	troubleHBoxLayout -> addWidget(troubleForumsButton);
	troubleHBoxLayout -> addWidget(troubleReportBugButton);
	troubleLabel -> setText("Please report bugs in our bug system, not in the forum");
	smallerFont.setPointSize(7);
	troubleLabel -> setFont(smallerFont);
	troubleVBoxLayout -> addWidget(troubleLabel);
	troubleVBoxLayout -> addLayout(troubleHBoxLayout);
	troubleGroupBox -> setLayout(troubleVBoxLayout);

	connect( troubleReadmeButton, SIGNAL( clicked() ), this, SLOT( readmeButtonClicked() ) );
	connect( troubleForumsButton, SIGNAL( clicked() ), this, SLOT( forumsButtonClicked() ) );
	connect( troubleReportBugButton, SIGNAL( clicked() ), this, SLOT( reportBugButtonClicked() ) );

	troubleReadmeButton -> setToolTip( GameInfo::getReadme() );
	troubleForumsButton -> setToolTip( GameInfo::getForums() );
	troubleReportBugButton -> setToolTip( GameInfo::getReportBug() );

	inGroupLayout -> addWidget(titleLabel);
	inGroupLayout -> addWidget(textLabel1);
	inGroupLayout -> addWidget(textLabel2);
	inGroupLayout -> setAlignment(titleLabel, Qt::AlignHCenter);
	inGroupLayout -> setAlignment(textLabel1, Qt::AlignHCenter);
	inGroupLayout -> addStretch();
	inGroupLayout -> addWidget(troubleGroupBox);
	groupBox -> setLayout(inGroupLayout);

	mainLayout -> addWidget(groupBox);
	mainLayout -> setContentsMargins(0, 0, 0, 0);
	setLayout(mainLayout);
}

void TabWelcome::readmeButtonClicked()
{
	System::runBrowser( GameInfo::getReadme() );
}

void TabWelcome::forumsButtonClicked()
{
	System::runBrowser( GameInfo::getForums() );
}

void TabWelcome::reportBugButtonClicked()
{
	System::runBrowser( GameInfo::getReportBug() );
}
