#include <windows.h>

#include "Registry.h"
#include "FilePaths.h"

#define MAX_LENGTH		1024

HKEY g_hkRoot = HKEY_LOCAL_MACHINE;

/* recursively delete registry entry */
bool regCleanDir( const QString & qstrLocation )
{
	/* temp variables */
	HKEY hKey;
	WCHAR keyName[ MAX_LENGTH ];	// buffer for key/value name
	DWORD keyLen;					// size of key/value string
	DWORD subKeys = 0;				// number of subkeys
	LONG retCode;
	int nSize;

	/* let's open our key */
	nSize = qstrLocation.toWCharArray( keyName );
	keyName[ nSize ] = '\0';

	retCode = RegOpenKeyEx(
				g_hkRoot,
				keyName,
				0,
				KEY_ALL_ACCESS,
				&hKey );

	if( retCode != ERROR_SUCCESS )
		return false;

	/* let's query our key */
	retCode = RegQueryInfoKey(
				hKey,			// key handle
				NULL,			// buffer for class name
				NULL,			// size of class string
				NULL,			// reserved
				&subKeys,		// number of subkeys
				NULL,			// longest subkey size
				NULL,			// longest class string
				NULL,			// number of values for this key
				NULL,			// longest value name
				NULL,			// longest value data
				NULL,			// security descriptor
				NULL );			// last write time

	if( retCode != ERROR_SUCCESS )
	{
		RegCloseKey( hKey );
		return false;
	}

	/* let's enumerate subkeys */
	for( unsigned int i = 0; i < subKeys; i++ )
	{
		keyLen = sizeof( keyName );
		retCode = RegEnumKeyEx(
					hKey,
					i,
					keyName,
					&keyLen,
					NULL,
					NULL,
					NULL,
					NULL );

		if( retCode != ERROR_SUCCESS )
		{
			RegCloseKey( hKey );
			return false;
		}

		QString qtrsNewLocation = qstrLocation + '\\' + QString::fromWCharArray( keyName );

		/* recurse call */
		if( !regCleanDir( qtrsNewLocation ) )
		{
			RegCloseKey( hKey );
			return false;
		}
	}

	/* all subkeys should be deleted, delete key */
	RegCloseKey( hKey );
	nSize = qstrLocation.toWCharArray( keyName );
	keyName[ nSize ] = '\0';
	retCode = RegDeleteKey(
				g_hkRoot,
				keyName );

	if( retCode != ERROR_SUCCESS )
		return false;

	return true;
}

/* recursively read registry entries */
bool regReadDir( QRegEntryList & qlReg, const QString & qstrLocation )
{
	/* temp variables */
	HKEY hKey;
	WCHAR keyName[ MAX_LENGTH ];	// buffer for key/value name
	DWORD keyLen;					// size of key/value string
	DWORD subKeys = 0;				// number of subkeys
	DWORD values = 0;				// number of values for key
	DWORD retCode;
	int nSize;

	/* find section name */
	QString qstrSection;
	if( qstrLocation == FILEPATHS.getRegistry() )
		qstrSection = "Default";
	else
	{
		qstrSection = qstrLocation;
		qstrSection.remove( FILEPATHS.getRegistry() + '\\' );

		/* premature recurse end, we don't need such deep keys */
		if( qstrSection.contains( '\\' ) )
			return true;
	}

	/* let's open our key */
	nSize = qstrLocation.toWCharArray( keyName );
	keyName[ nSize ] = '\0';

	retCode = RegOpenKeyEx(
				g_hkRoot,
				keyName,
				0,
				KEY_READ,
				&hKey );

	if( retCode != ERROR_SUCCESS )
		return false;

	/* let's query our key */
	retCode = RegQueryInfoKey(
				hKey,			// key handle
				NULL,			// buffer for class name
				NULL,			// size of class string
				NULL,			// reserved
				&subKeys,		// number of subkeys
				NULL,			// longest subkey size
				NULL,			// longest class string
				&values,		// number of values for this key
				NULL,			// longest value name
				NULL,			// longest value data
				NULL,			// security descriptor
				NULL );			// last write time

	if( retCode != ERROR_SUCCESS )
	{
		RegCloseKey( hKey );
		return false;
	}

	/* let's enumerate subkeys */
	for( unsigned int i = 0; i < subKeys; i++ )
	{
		keyLen = sizeof( keyName );
		retCode = RegEnumKeyEx(
					hKey,
					i,
					keyName,
					&keyLen,
					NULL,
					NULL,
					NULL,
					NULL );

		if( retCode != ERROR_SUCCESS )
		{
			RegCloseKey( hKey );
			return false;
		}

		QString qtrsNewLocation = qstrLocation + '\\' + QString::fromWCharArray( keyName );

		/* recurse call */
		if( !regReadDir( qlReg, qtrsNewLocation ) )
		{
			RegCloseKey( hKey );
			return false;
		}
	}

	/* let's enumerate and fill values */
	for( unsigned int i = 0; i < values; i++ )
	{
		DWORD keyType;
		BYTE data[ MAX_LENGTH * sizeof( WCHAR ) ];		// buffer for data
		DWORD dataLen = sizeof( data );					// size of data

		keyLen = sizeof( keyName );
		retCode = RegEnumValue(
					hKey,
					i,
					keyName,
					&keyLen,
					NULL,
					&keyType,
					data,
					&dataLen );

		if( retCode != ERROR_SUCCESS )
		{
			RegCloseKey( hKey );
			return false;
		}

		QString qstrKey = qstrSection + ':' + QString::fromWCharArray( keyName );
		QString qstrVal;
		ERegEntryType eret;

		switch( keyType )
		{
		case REG_SZ:
			qstrVal = QString::fromWCharArray( ( WCHAR *) data );
			eret = eretString;
			break;
		case REG_DWORD:
			qstrVal = QString( "%1" ).arg( ( ( DWORD * )data )[ 0 ] );
			eret = eretDWORD;
			break;
		default:
			break;	// ignore other types
		}

		CRegEntry re;
		re.m_qstrKey = qstrKey;
		re.m_qstrVal = qstrVal;
		re.m_eretType = eret;

		qlReg.append( re );
	}

	RegCloseKey( hKey );
	return true;
}

bool Registry::readRegistry()
{
	bool bRet = regReadDir( m_qlRegistrySaved, FILEPATHS.getRegistry() );

	if( bRet )
		qSort( m_qlRegistrySaved );
	else
		m_qlRegistrySaved.clear();

	return bRet;
}

bool Registry::saveRegistry()
{
	/*
	 * remove all entries so the ones we removed
	 * internally in Registry will really get deleted
	 * ignore return code, it can be file_not_found, permissions_denied, etc...
	 * in worst case, let's try to save what we got anyway
	 */
	regCleanDir( FILEPATHS.getRegistry() );

	for( int i = 0; i < m_qlRegistry.size(); i++ )
	{
		QString qstrSection = m_qlRegistry.at( i ).m_qstrKey.section( ':', 0, 0 );
		QString qstrKey = m_qlRegistry.at( i ).m_qstrKey.section( ':', 1, 1 );
		QString qstrPath = FILEPATHS.getRegistry();
		if( qstrSection != "Default" )
			qstrPath += '\\' + qstrSection;

		int nSize;
		WCHAR path[ qstrPath.size() + 1 ];
		nSize = qstrPath.toWCharArray( path );
		path[ nSize ] = '\0';

		WCHAR keyName[ qstrKey.size() + 1 ];
		nSize = qstrKey.toWCharArray( keyName );
		keyName[ nSize ] = '\0';

		/* get handle creating the key if necessary */
		DWORD retCode;
		HKEY hKey;

		retCode = RegCreateKeyEx(
					g_hkRoot,
					path,
					0,
					NULL,
					REG_OPTION_NON_VOLATILE,
					KEY_WRITE,
					NULL,
					&hKey,
					NULL );

		if( retCode != ERROR_SUCCESS )
			return false;

		/* prepare and write data */
		BYTE data[ MAX_LENGTH * sizeof( WCHAR ) ];
		DWORD dataLen;
		DWORD dataType;
		WCHAR * wData = ( WCHAR * )data;
		DWORD * dData = ( DWORD * )data;

		switch( m_qlRegistry.at( i ).m_eretType )
		{
		case eretString:
			// FIXME: possible buffer overflow
			nSize = m_qlRegistry.at( i ).m_qstrVal.toWCharArray( wData );
			wData[ nSize ] = '\0';
			dataType = REG_SZ;
			dataLen = sizeof( WCHAR ) * ( nSize + 1 );
			break;
		case eretDWORD:
			dData[ 0 ] = m_qlRegistry.at( i ).m_qstrVal.toInt();
			dataType = REG_DWORD;
			dataLen = sizeof( DWORD );
			break;
		default:
			break;		// ignore other types
		}

		retCode = RegSetValueEx(
					hKey,
					keyName,
					0,
					dataType,
					data,
					dataLen );

		if( retCode != ERROR_SUCCESS )
		{
			RegCloseKey( hKey );
			return false;
		}

		RegCloseKey( hKey );
	}

	return true;
}
