#include <windows.h>
#include <regstr.h>
#include <servprov.h>
#include <sapi.h>
//#include <sphelper.h>

#include "System.h"

#define MAX_LENGTH		1024

/********************************** MISC **********************************/

bool System::run( const QString & qstrExec, const QStringList & qstrlParams, bool bWaitForEnd )
{
	if( qstrExec.isEmpty() )
		return false;

	if( QFileInfo( qstrExec ).isAbsolute() )
	{
		if ( ! QFileInfo( qstrExec ).exists() )
			return false;
		if ( ! QFileInfo( qstrExec ).isExecutable() )
			return false;
	}

	WCHAR appName[ qstrExec.size() + 1 ];
	int nSize = qstrExec.toWCharArray( appName );
	appName[ nSize ] = '\0';

	QString qstrCmd;
	for( int i = 0; i < qstrlParams.size(); i++ )
	{
		qstrCmd.append( " " );
		qstrCmd.append( qstrlParams.at( i ) );
	}
	WCHAR cmdLine[ qstrCmd.size() + 1 ];
	nSize = qstrCmd.toWCharArray( cmdLine );
	cmdLine[ nSize ] = '\0';

	PROCESS_INFORMATION process_info;
	STARTUPINFO startup_info;

	memset( &startup_info, 0, sizeof( STARTUPINFO ) );
	startup_info.cb = sizeof( STARTUPINFO );

	SetErrorMode( SEM_NOOPENFILEERRORBOX );

	bool bResult = CreateProcess(
				appName,
				cmdLine,
				NULL, NULL, FALSE,
				CREATE_NEW_CONSOLE | CREATE_DEFAULT_ERROR_MODE,	// flags
				NULL,
				NULL,
				&startup_info,
				&process_info);

	if( bResult == false )
		return false;

	if( bWaitForEnd )
	{
		DWORD exit_code;
		do {
			// Sleep so we dont query madly and slow everything down
			Sleep( 20 );
			GetExitCodeProcess( process_info.hProcess, &exit_code );
		} while( exit_code == STILL_ACTIVE );
	}

	return true;
}

bool System::runBrowser( const QString & qstrUrl )
{
	if( qstrUrl.isEmpty() )
		return false;

	WCHAR wszUrl[ qstrUrl.size() + 1 ];
	int nSize = qstrUrl.toWCharArray( wszUrl );
	wszUrl[ nSize ] = '\0';

	ShellExecute( 0, L"open", wszUrl, 0, 0, SW_SHOWDEFAULT );

	return true;
}

int System::mSleep( int nMsec )
{
	Sleep( nMsec );
	return 1;
}

QSize System::getSize()
{
	return QSize( 350, 500 );
}

/********************************** VIDEO **********************************/

struct CVidMode
{
	int m_nX;
	int m_nY;

	CVidMode( int nX, int nY ) :
		m_nX( nX ), m_nY ( nY )	{}
};

bool operator<( const CVidMode & v1, const CVidMode & v2 )
{
	if( v1.m_nX != v2.m_nX )
		return v1.m_nX < v2.m_nX;

	return v1.m_nY < v2.m_nY;
}

bool operator==( const CVidMode & v1, const CVidMode & v2 )
{
	if( v1.m_nX != v2.m_nX )
		return false;

	if( v1.m_nY != v2.m_nY )
		return false;

	return true;
}

QStringList System::enumerateResolutions()
{
	QList< CVidMode > qlVids;
	QStringList qstrl;

	DEVMODE dm;
	DWORD n = 0;

	dm.dmSize = sizeof( DEVMODE );
	dm.dmDriverExtra = 0;

	while( EnumDisplaySettings( NULL, n++, &dm ) )
	{
		if( dm.dmPelsWidth < 640 )
			continue;
		if( dm.dmPelsHeight < 480 )
			continue;
		if( dm.dmBitsPerPel < 16 )
			continue;

		CVidMode vm( dm.dmPelsWidth, dm.dmPelsHeight );

		if( qlVids.contains( vm ) )
			continue;

		qlVids.append( vm );
	}

	qSort( qlVids );

	for( int i = qlVids.size() - 1; i >= 0 ; i-- )
	{
		QString qstrMode = QString( "%1x%2" ).arg( qlVids.at( i ).m_nX ).arg( qlVids.at( i ).m_nY );
		qstrl.append( qstrMode );
	}

	return qstrl;
}

QList< int > System::enumerateBPP()
{
	QList< int > ql;

	DEVMODE dm;
	DWORD n = 0;

	dm.dmSize = sizeof( DEVMODE );
	dm.dmDriverExtra = 0;

	while( EnumDisplaySettings( NULL, n++, &dm ) )
	{
		if( dm.dmPelsWidth < 640 )
			continue;
		if( dm.dmPelsHeight < 480 )
			continue;
		if( dm.dmBitsPerPel < 16 )
			continue;

		if( ql.contains( dm.dmBitsPerPel ) )
			continue;

		ql.append( dm.dmBitsPerPel );
	}

	qSort( ql );
	return ql;
}

/********************************** AUDIO **********************************/

/* #include <alc.h> :-D */
#define ALC_DEFAULT_DEVICE_SPECIFIER	0x1004
#define ALC_DEVICE_SPECIFIER			0x1005
typedef struct ALCdevice_struct ALCdevice;
typedef char ALCchar;
typedef int ALCenum;

/* function type, easier that way */
typedef const ALCchar* ( *TalcGetString )( ALCdevice *device, ALCenum param );

/* based on CTabSound::SetupOpenAL from MFC FS2 launcher */
QStringList System::enumerateAudioDevices()
{
	QStringList qstrl;

	HINSTANCE hOAL;
	TalcGetString alcGetString = NULL;

	if( ( hOAL = LoadLibrary( L"OpenAL32.dll" ) ) == 0 )
	{
		System::warning( "Build needs OpenAL but an OpenAL DLL cannot be found!" );
		return qstrl;
	}

	alcGetString = ( TalcGetString )GetProcAddress( hOAL, "alcGetString" );

	if( alcGetString == NULL )
	{
		FreeLibrary( hOAL );
		System::warning( "Unable to acquire alcGetString pointer!" );
		return qstrl;
	}

	/* 0 separated list of devices terminated with two 0 characters */
	const char * devices = ( const char* )alcGetString( NULL, ALC_DEVICE_SPECIFIER );

	if( devices == NULL )
		return qstrl;

	char * pos = ( char * )devices;

	while( *pos /*&& (strlen(pos) > 0 )*/ )
	{
		qstrl.append( QString( pos ) );
		pos += ( strlen( pos ) + 1 );
	}

	FreeLibrary( hOAL );

	return qstrl;
}

QString System::getGenericDevice()
{
	return "Generic Software";
}

/********************************** INPUT **********************************/

/* based on CTabSound::OnInitDialog() from MFC FS2 launcher */
QStringList System::enumerateJoystickDevices()
{
	QStringList qstrl;

	int nJoyNum = joyGetNumDevs();

	for( int i = 0; i < nJoyNum; i++ )
	{
		JOYCAPS jc;
		JOYINFO ji;
		WCHAR keyName[ MAX_LENGTH ];
		WCHAR valueName[ MAX_LENGTH ];
		WCHAR data[ MAX_LENGTH ];
		HKEY hKey;
		DWORD len;
		LONG retCode;
		QString qstrJoyName;

		memset( &jc, 0, sizeof( JOYCAPS ) );
		MMRESULT mr = joyGetDevCaps( i, &jc, sizeof( JOYCAPS ) );

		// make sure that our device/driver is good
		if( mr != JOYERR_NOERROR )
			continue;

		// if the joystick is unplugged or otherwise not available then just ignore it
		if( joyGetPos( i, &ji ) != JOYERR_NOERROR )
			continue;

		// copy basic device name, in case getting the OEM name from the registry fails
		qstrJoyName = QString::fromWCharArray( jc.szPname );

		// now try to grab the full OEM name for this joystick ...
		swprintf( keyName, L"%s\\%s\\%s", REGSTR_PATH_JOYCONFIG, jc.szRegKey, REGSTR_KEY_JOYCURR );
		retCode = RegOpenKeyEx( HKEY_LOCAL_MACHINE, keyName, 0, KEY_QUERY_VALUE, &hKey );

		if( retCode != ERROR_SUCCESS )
			goto done;

		len = sizeof( data );
		// NOTE: normal joystick values are 0 based, but it's 1 based in the registry, hence the "+1"
		swprintf( valueName, L"Joystick%d%s", i + 1, REGSTR_VAL_JOYOEMNAME );
		retCode = RegQueryValueEx( hKey, valueName, 0, 0, ( BYTE * ) data, &len );
		RegCloseKey( hKey );

		if( retCode != ERROR_SUCCESS )
			goto done;

		swprintf( keyName, L"%s\\%s", REGSTR_PATH_JOYOEM, data );
		retCode = RegOpenKeyEx( HKEY_LOCAL_MACHINE, keyName, 0, KEY_QUERY_VALUE, &hKey );

		if( retCode != ERROR_SUCCESS )
			goto done;

		// grab the OEM name
		len = sizeof( valueName );
		retCode = RegQueryValueEx( hKey, REGSTR_VAL_JOYOEMNAME, 0, 0, ( BYTE * ) data, &len );
		RegCloseKey( hKey );

		if( retCode != ERROR_SUCCESS )
			goto done;

		qstrJoyName = QString::fromWCharArray( data );

done:
		qstrl.append( qstrJoyName );
	}

	return qstrl;
}

void System::runCalibrateDialog( const int& )
{
	//QStringList qstrlParams;
	//qstrlParams.append( "shell32.dll,Control_RunDLL" );
	//qstrlParams.append( "joy.cpl" );
	//System::run( "rundll32.exe", qstrlParams );

	WinExec( "rundll32.exe shell32.dll,Control_RunDLL joy.cpl", SW_SHOWNORMAL );
}

/********************************** SPEECH **********************************/

bool g_bSpeech = false;
ISpVoice * g_pVoiceDevice = NULL;
ISpObjectTokenCategory * g_pTokenCategory = NULL;
IEnumSpObjectTokens * g_pEnum = NULL;


void System::speechInit()
{
	HRESULT hr = CoCreateInstance(
					CLSID_SpVoice,
					NULL,
					CLSCTX_ALL,
					IID_ISpVoice,
					( void ** )&g_pVoiceDevice );

	if( !SUCCEEDED( hr ) )
		goto fail;

	hr = CoCreateInstance(
			CLSID_SpObjectTokenCategory,
			NULL,
			CLSCTX_ALL,
			IID_ISpObjectTokenCategory,
			( void **)&g_pTokenCategory );

	if( !SUCCEEDED( hr ) )
		goto fail;

	hr = g_pTokenCategory->SetId( SPCAT_VOICES, false );

	if( !SUCCEEDED( hr ) )
		goto fail;

	hr = g_pTokenCategory->EnumTokens(
			NULL,
			NULL,
			&g_pEnum );

	if( !SUCCEEDED( hr ) )
		goto fail;

	g_bSpeech = true;
	return;

fail:
	if( g_pVoiceDevice != NULL )
	{
		g_pVoiceDevice->Release();
		g_pVoiceDevice = NULL;
	}
	if( g_pTokenCategory != NULL )
	{
		g_pTokenCategory->Release();
		g_pTokenCategory = NULL;
	}
	if( g_pEnum != NULL )
	{
		g_pEnum->Release();
		g_pEnum = NULL;
	}
	g_bSpeech = false;
}

void System::speechDeinit()
{
	if( !g_bSpeech ) return;

	g_pVoiceDevice->Release();
	g_pVoiceDevice = NULL;
	g_pTokenCategory->Release();
	g_pTokenCategory = NULL;
	g_pEnum->Release();
	g_pEnum = NULL;

	g_bSpeech = false;
}

bool System::speechReady()
{
	return g_bSpeech;
}

QStringList System::enumerateVoices()
{
	QStringList qstrl;
	if( !g_bSpeech ) return qstrl;

	HRESULT hr;
	ISpObjectToken * pToken;
	g_pEnum->Reset();

	while( g_pEnum->Next( 1, &pToken, NULL ) == S_OK )
	{
		WCHAR * name;
		hr = pToken->GetStringValue(NULL, &name);
		if( SUCCEEDED( hr ) )
			qstrl.append( QString::fromWCharArray( name ) );

		if( pToken != NULL )
			pToken->Release();
	}

	return qstrl;
}

void System::speechSetVoice( const int & nVoice )
{
	if( !g_bSpeech ) return;

	HRESULT hr;
	ISpObjectToken * pToken;
	g_pEnum->Reset();

	hr = g_pEnum->Item( nVoice, &pToken );
	if( SUCCEEDED( hr ) )
		g_pVoiceDevice->SetVoice( pToken );

	if( pToken != NULL )
		pToken->Release();
}

void System::speechSetVolume( const int & nVolume )
{
	if( !g_bSpeech ) return;

	g_pVoiceDevice->SetVolume( nVolume );
}

void System::speechPlay( const QString & qstr )
{
	if( !g_bSpeech ) return;

	if( qstr.isEmpty() )
		return;

	WCHAR text[ qstr.size() + 1 ];
	int nSize = qstr.toWCharArray( text );
	text[ nSize ] = '\0';

	speechStop();
	g_pVoiceDevice->Speak( text, SPF_ASYNC, NULL );
}

void System::speechStop()
{
	g_pVoiceDevice->Speak( NULL, SPF_PURGEBEFORESPEAK, NULL );
}
